/* printq.c -- source file for printq.dll       */
/* (C) Peter Wansch und Siegfried Hanisch, 1993 */
/* created: 93-5-7                              */

#include <stddef.h>
#include <string.h> 
#include "printq.h"

/* constants */
#define LEN_WORKSTRING 256
#define LEN_BUFFER 1024

/* global variables */
HAB habApp;
HWND hwndList;
HDC  hdcPrn;
HPS hpsPrn;
PVOID  pBuf;
DEVOPENSTRUC dopData;
ULONG  ulDefIndex;
ULONG ulNumOfQs;
BOOL fInitialized = FALSE;
BOOL fJobStarted = FALSE;
BOOL fListDisplayed = FALSE;
BOOL fQueryInfo = FALSE;

/* exported global varaibles */
PRQINFO3 prqInfoDef;
HCINFO hcInfoDef; 
LONG alCaps[CAPS_LINEWIDTH_THICK]; 

BOOL SpoolInitialize(HAB hab)
{
  PPRQINFO3 prq;
  PHCINFO phcinfo;
  CHAR szBuffer[LEN_BUFFER+1];
  SPLERR splerr;
  CHAR   szDefQName[LEN_WORKSTRING]; 
  ULONG  cbBuf ;
  ULONG  cTotal;
  ULONG  cbNeeded ;
  ULONG  i;
  PCHAR  pch;
  PVOID  pBuffer;
  LONG clForms;

  if (fInitialized || fJobStarted || fListDisplayed || fQueryInfo)
    return FALSE;
  habApp = hab;
  if (PrfQueryProfileString(HINI_PROFILE, "PM_SPOOLER", "QUEUE", NULL, szDefQName, LEN_WORKSTRING))
  {  
    pch = strchr(szDefQName, ';');
    if (pch)
      *pch = 0;
  }
  else
    return FALSE;
  
  splerr = SplEnumQueue((PSZ)NULL, 3L, pBuf, 0L, &ulNumOfQs, &cTotal, &cbNeeded, NULL);
  if (splerr != ERROR_MORE_DATA)
    return FALSE;
  if (DosAllocMem(&pBuf, cbNeeded, PAG_READ|PAG_WRITE|PAG_COMMIT))
    return FALSE;
  cbBuf = cbNeeded;
  splerr = SplEnumQueue((PSZ)NULL, 3L, pBuf, cbBuf, &ulNumOfQs, &cTotal, &cbNeeded, NULL);
  if (splerr != NO_ERROR)
  {
    DosFreeMem(pBuf);
    return FALSE;
  }
  prq = (PPRQINFO3)pBuf;        
  /* ulNumOfQs has the count of the number of PRQINFO3 structures */
  i = 0; /* index */
  while (strcmp(prq->pszName, szDefQName)!=0 && i < ulNumOfQs)
  {
    i++;
    prq++;   
  }
  if (i == ulNumOfQs)
  {
    DosFreeMem(pBuf);
    return FALSE;
  }
  /* prq points to default PRQINFO3 structure */
  ulDefIndex = i;
  prqInfoDef = *prq;
  /* fill in device open structure */
  dopData.pszLogAddress = prq->pszName;
  dopData.pdriv = prq->pDriverData;
  strcpy(szBuffer, prq->pszDriverName);
  pch = strchr(szBuffer, '.');
  if (pch)
    *pch = 0;
  dopData.pszDriverName = szBuffer;
  hdcPrn = DevOpenDC(habApp, OD_INFO, "*", 3L, (PDEVOPENDATA)&dopData, (HDC)0);
  if (hdcPrn == DEV_ERROR)
  {
    DosFreeMem(pBuf);
    return FALSE;
  }
  /* now we obtain some device information */
  if (!DevQueryCaps(hdcPrn, CAPS_FAMILY, CAPS_LINEWIDTH_THICK, alCaps))
  {
    DevCloseDC(hdcPrn);
    DosFreeMem(pBuf);
    return FALSE;
  }
  /* now we obtain information on the default form */
  clForms = DevQueryHardcopyCaps(hdcPrn, 0L, 0L, NULL);
  if (clForms == DQHC_ERROR)
  {
    DevCloseDC(hdcPrn);
    DosFreeMem(pBuf);
    return FALSE;
  }
  if (DosAllocMem(&pBuffer, clForms*sizeof(HCINFO), PAG_READ|PAG_WRITE|PAG_COMMIT)!=NO_ERROR)
  {
    DevCloseDC(hdcPrn);
    DosFreeMem(pBuf);
    return FALSE;
  }
  phcinfo = (PHCINFO)pBuffer;
  if (DevQueryHardcopyCaps(hdcPrn, 0L, clForms, phcinfo) == DQHC_ERROR)
  {
    DosFreeMem(pBuffer); 
    DevCloseDC(hdcPrn);
    DosFreeMem(pBuf); 
    return FALSE;
  }          
  while (!(phcinfo->flAttributes & HCAPS_CURRENT)) 
    phcinfo++;
  /* phcinfo now points to the HCINFO for the current form */
  hcInfoDef = *phcinfo;
  DosFreeMem(pBuffer);
  DevCloseDC(hdcPrn);
  fInitialized = TRUE;
  return TRUE;
}

BOOL SpoolBeginInfo(PHPS phpsPrinterInfo)
{
  SIZEL szl;
  PPRQINFO3 prq;
  CHAR szBuffer[LEN_BUFFER+1];
  PCHAR  pch;

  if (!(fInitialized && !fListDisplayed && !fJobStarted && !fQueryInfo))
    return FALSE;
  prq = (PPRQINFO3) pBuf;
  prq += ulDefIndex;
  /* fill in device open structure */
  strcpy(szBuffer, prq->pszDriverName);
  pch = strchr(szBuffer, '.');
  if (pch)
    *pch = 0;
  dopData.pszDriverName = szBuffer;
  hdcPrn = DevOpenDC(habApp, OD_INFO, "*", 3L, (PDEVOPENDATA)&dopData, (HDC)0);
  if (hdcPrn == DEV_ERROR)
    return FALSE;
  szl.cx=0L;
  szl.cy=0L;
  hpsPrn = GpiCreatePS(habApp, hdcPrn, &szl, PU_PELS | GPIA_ASSOC);
  if (hpsPrn == GPI_ERROR)
  {
    DevCloseDC(hdcPrn);
    return FALSE;
  }
  *phpsPrinterInfo = hpsPrn;
  fQueryInfo = TRUE;
  return TRUE;
}

BOOL SpoolEndInfo(void)
{
  if (!(fInitialized && !fListDisplayed && !fJobStarted && fQueryInfo))
    return FALSE;
  GpiDestroyPS(hpsPrn);
  DevCloseDC(hdcPrn);
  fQueryInfo = FALSE;
  return TRUE;
}

BOOL SpoolSetDef(void)
{
  SHORT index;  
  PPRQINFO3 prq;
  PHCINFO phcinfo;
  CHAR szBuffer[LEN_BUFFER+1];
  LONG clForms;
  PVOID  pBuffer;
  PCHAR  pch;
  PRQINFO3 prqInfoDefTemp;
  ULONG ulDefIndexTemp;
  LONG alCapsTemp[CAPS_LINEWIDTH_THICK]; 

  if (!(fInitialized && fListDisplayed && !fJobStarted && !fQueryInfo))
    return FALSE;
  index = WinQueryLboxSelectedItem(hwndList);
  if (index == LIT_NONE)
    return FALSE;
  prq = (PPRQINFO3) pBuf;
  prq += index;
  prqInfoDefTemp = *prq;
  ulDefIndexTemp = (ULONG)index;
  /* fill in device open structure */
  dopData.pszLogAddress = prq->pszName;
  dopData.pdriv = prq->pDriverData;
  strcpy(szBuffer, prq->pszDriverName);
  pch = strchr(szBuffer, '.');
  if (pch)
    *pch = 0;
  dopData.pszDriverName = szBuffer;
  hdcPrn = DevOpenDC(habApp, OD_INFO, "*", 3L, (PDEVOPENDATA)&dopData, (HDC)0);
  if (hdcPrn == DEV_ERROR)
    return FALSE;
  /* now we obtain some device information */
  if (!DevQueryCaps(hdcPrn, CAPS_FAMILY, CAPS_LINEWIDTH_THICK, alCapsTemp))
  {
    DevCloseDC(hdcPrn);
    return FALSE;
  }
  /* now we obtain information on the default form */
  clForms = DevQueryHardcopyCaps(hdcPrn, 0L, 0L, NULL);
  if (clForms == DQHC_ERROR)
  {
    DevCloseDC(hdcPrn);
    return FALSE;
  }
  if (DosAllocMem(&pBuffer, clForms*sizeof(HCINFO), PAG_READ|PAG_WRITE|PAG_COMMIT)!=NO_ERROR)
  {
    DevCloseDC(hdcPrn);
    return FALSE;
  }
  phcinfo = (PHCINFO)pBuffer;
  if (DevQueryHardcopyCaps(hdcPrn, 0L, clForms, phcinfo) == DQHC_ERROR)
  {
    DosFreeMem(pBuffer); 
    DevCloseDC(hdcPrn);
    return FALSE;
  }          
  while (!(phcinfo->flAttributes & HCAPS_CURRENT))
    phcinfo++;
  /* phcinfo now points to the HCINFO for the current form */

  hcInfoDef = *phcinfo;
  prqInfoDef = prqInfoDefTemp;
  ulDefIndex = ulDefIndexTemp;
  for (index = 0; index < CAPS_LINEWIDTH_THICK; index++)
    alCaps[index] = alCapsTemp[index]; 
  
  DosFreeMem(pBuffer);
  DevCloseDC(hdcPrn);
  return TRUE;
}

BOOL SpoolBeginSetup(HWND hwndListBox)
{
  PPRQINFO3 prq;
  PSZ psz;
  SHORT i,index;
  hwndList = hwndListBox;
  if (!(fInitialized && !fJobStarted && !fListDisplayed && !fQueryInfo))
    return FALSE;
  if (!WinSendMsg(hwndList, LM_DELETEALL, (MPARAM)NULL, (MPARAM)NULL))
    return FALSE;
  prq = (PPRQINFO3) pBuf;
  /* prq points to the first PRQINFO3 structure */
  for (i = 0; i < ulNumOfQs; i++) 
    {
      psz = *prq->pszComment ? prq->pszComment : prq->pszName;
      index = WinInsertLboxItem(hwndList, LIT_END, psz);
      if (index == LIT_MEMERROR || index == LIT_ERROR || index != i)
	return FALSE;
      prq++;
    }
  if (!WinSendMsg(hwndList, LM_SELECTITEM, (MPARAM)ulDefIndex, (MPARAM)TRUE))
    return FALSE;
  fListDisplayed = TRUE;
  return TRUE;
}

BOOL SpoolJobProp(void)
{
  PPRQINFO3 prq;
  LONG cbBuf;
  PSZ pszDeviceName, psz;
  SHORT index;
  CHAR szBuffer[LEN_BUFFER+1];
  
  if (!(fInitialized && fListDisplayed && !fJobStarted && !fQueryInfo))
    return FALSE;
  index = WinQueryLboxSelectedItem(hwndList);
  if (index == LIT_NONE)
    return FALSE;
  prq = (PPRQINFO3) pBuf;
  prq += index;

  psz = strchr(prq->pszPrinters, ',');
  if (psz)
    *psz=0;
  strcpy(szBuffer, prq->pszDriverName);
  pszDeviceName = strchr(szBuffer, '.');
  if (pszDeviceName)
  {
    *pszDeviceName=0;
    pszDeviceName++;
  }

  cbBuf = DevPostDeviceModes(habApp, (PDRIVDATA)NULL, szBuffer, pszDeviceName, prq->pszPrinters, DPDM_POSTJOBPROP);
  if (cbBuf<=0 || cbBuf != prq->pDriverData->cb)
    return FALSE; 
  cbBuf = DevPostDeviceModes(habApp, prq->pDriverData, szBuffer, pszDeviceName, prq->pszPrinters, DPDM_POSTJOBPROP);
  if (cbBuf == DPDM_ERROR)
    return FALSE;
  /* the Job properties of the default printer might have been edited */
  if ((ULONG)index == ulDefIndex)
    if (!SpoolSetDef())
      return FALSE; 
  return TRUE;
}

BOOL SpoolBeginSpool(PSZ pszComment, PSZ pszDocName, PSZ pszQueueProcParams, ULONG ulOptions, PHPS phpsPrinter)
{
  SIZEL szl;
  PPRQINFO3 prq;
  CHAR szBuffer[LEN_BUFFER+1];
  PCHAR  pch;
  
  if (!(fInitialized && !fJobStarted && !fListDisplayed && !fQueryInfo))
    return FALSE;
  /* SpoolStart starts a print job on the default queue and returns hpsPrn */
  prq = (PPRQINFO3) pBuf;
  prq += ulDefIndex;
  strcpy(szBuffer, prq->pszDriverName);
  pch = strchr(szBuffer, '.');
  if (pch)
    *pch = 0;
  dopData.pszDriverName = szBuffer;
  dopData.pszDataType="PM_Q_STD";
  dopData.pszComment=pszComment;
  dopData.pszQueueProcName=NULL;
  dopData.pszQueueProcParams=pszQueueProcParams; 
  dopData.pszSpoolerParams=NULL;
  dopData.pszNetworkParams=NULL;
  hdcPrn = DevOpenDC (habApp, OD_QUEUED, "*", 9L, (PDEVOPENDATA)&dopData, (HDC)0);
  if (hdcPrn == DEV_ERROR)
    return FALSE;
  szl.cx=0L;
  szl.cy=0L;
  if (ulOptions == 0L)
    ulOptions = PU_PELS;
  hpsPrn = GpiCreatePS(habApp, hdcPrn, &szl, ulOptions | GPIA_ASSOC);
  if (hpsPrn == GPI_ERROR)
  {
    DevCloseDC(hdcPrn);
    return FALSE;
  }
  *phpsPrinter = hpsPrn;
  if (DevEscape(hdcPrn, DEVESC_STARTDOC, (LONG)sizeof(pszDocName), (PBYTE)pszDocName, (LONG)0, (PBYTE)NULL)!=DEV_OK)
  {
    GpiDestroyPS(hpsPrn);
    DevCloseDC(hdcPrn);
    return FALSE;
  }
  fJobStarted = TRUE;
  return TRUE;
}

BOOL SpoolTerminate(void)
{
  if (!(fInitialized && !fJobStarted && !fListDisplayed && !fQueryInfo))
    return FALSE;
  DosFreeMem(pBuf);
  fInitialized = FALSE;
  return TRUE;
}

BOOL SpoolNewFrame(void)
{
  if (!(fInitialized && !fListDisplayed && fJobStarted && !fQueryInfo))
    return FALSE;
  if (DevEscape (hdcPrn, DEVESC_NEWFRAME, 0L, NULL, 0L, NULL) == DEV_OK) 
    return TRUE;
  else
    return FALSE;
}

BOOL SpoolEndSpool(PUSHORT pusJobId)
{
  LONG loutData=2L;
  if (!(fInitialized && fJobStarted && !fListDisplayed && !fQueryInfo))
    return FALSE;
  DevEscape (hdcPrn, DEVESC_ENDDOC, (LONG)0, (PBYTE)NULL, &loutData, (PBYTE)pusJobId);
  GpiDestroyPS(hpsPrn);
  DevCloseDC(hdcPrn);
  fJobStarted = FALSE;
  return TRUE;
}

BOOL SpoolAbortSpool(void)
{
  if (!(fInitialized && fJobStarted && !fListDisplayed && !fQueryInfo))
    return FALSE;
  DevEscape (hdcPrn, DEVESC_ABORTDOC, (LONG)0, (PBYTE)NULL, (PLONG)0L, (PBYTE)NULL);
  GpiDestroyPS(hpsPrn);
  DevCloseDC(hdcPrn);
  fJobStarted = FALSE;
  return TRUE;
}

BOOL SpoolEndSetup(void)
{
  if (!(fInitialized && !fJobStarted && fListDisplayed && !fQueryInfo))
    return FALSE;
  fListDisplayed = FALSE;
  return TRUE;
}
